import {Camera, math, Viewer} from '@xeokit/xeokit-sdk';
import {BimShowTime} from './index';

/**
 * 3D浏览器相机相关操作类
 */
export class BstCamera {
    viewer: Viewer;
    camera: Camera;

    constructor(private bst: BimShowTime) {
        this.viewer = bst.viewer;
        this.camera = this.viewer.camera;
    }

    /**
     * 根据给出的构件ID列表,调整画面显示
     * @param objectIds 构件ID列表
     */
    viewFit(objectIds: Array<string>): void {
        this.viewer.cameraFlight.flyTo({
            projection: 'perspective',
            aabb: this.viewer.scene.getAABB(objectIds),
            duration: 0.5
        }, () => {
            setTimeout(() => {
                this.viewer.scene.setObjectsHighlighted(this.viewer.scene.highlightedObjectIds, false);
            }, 500);
        })
    }

    /**
     * 重置视角
     * 根据所有已经加载的模型,调整一个能包含全部的相机位置
     */
    reset(): Promise<boolean> {
        return new Promise<boolean>((resolve, reject) => {
            const scene = this.viewer.scene;
            const aabb = scene.getAABB(scene.visibleObjectIds);
            const diag = math.getAABB3Diag(aabb);
            // @ts-ignore
            const center = math.getAABB3Center(aabb, math.vec3());
            const camera = scene.camera;
            const fitFOV = camera.perspective.fov;
            const up = math.normalizeVec3((camera.yUp) ? [-0, 0.7071, -0.5] : [-1, 1, 1]);
            this.viewer.cameraControl.pivotPos = center;
            this.viewer.cameraControl.navMode = 'orbit';
            this.viewer.cameraFlight.flyTo({
                look: center,
                up,
                orthoScale: diag * 1.3,
                projection: 'perspective',
                duration: 1
            });
            resolve(true);
        })
    }

    /**
     * 获取相机的距离范围
     * @return min为最小距离,max为最大距离
     */
    getDistanceConfig(): { min: number, max: number } {
        return {min: this.camera.frustum.near, max: this.camera.frustum.far}
    }

    /**
     * 设置相机的距离范围
     * @param min 最小距离
     * @param max 最大距离
     */
    setDistanceConfig(min: number, max: number): void {
        this.camera.frustum.near = min;
        this.camera.frustum.far = max;
    }

    /**
     * 获取当前缩放速度
     */
    getScalingSpeed(): number {
        return this.viewer.cameraControl.dollyMinSpeed;
    }


    /**
     * 设置缩放速度
     * @param speed 缩放速度
     */
    setScalingSpeed(speed: number) {
        this.viewer.cameraControl.dollyMinSpeed = speed;
    }

    /**
     * 获取当前相机的三维信息
     */
    getPosition(): { x: number, y: number, z: number } {
        return {x: this.viewer.camera.eye[0], y: this.viewer.camera.eye[1], z: this.viewer.camera.eye[2]}
    }

    /**
     * 设置当前相机的三维信息
     * @param x
     * @param y
     * @param z
     */
    setPosition(x: number, y: number, z: number) {
        this.viewer.camera.eye = [x, y, z];
    }

    /**
     * 是否启用平移功能
     * @param enable true是启用,false为禁用
     */
    enableTranslation(enable: boolean = true): void {
        if (enable) {
            this.viewer.cameraControl.touchPanRate = 1;
            this.viewer.cameraControl.keyboardPanRate = 5;
            this.viewer.cameraControl.panInertia = 0.5;
        } else {
            this.viewer.cameraControl.touchPanRate = 0;
            this.viewer.cameraControl.keyboardPanRate = 0;
            this.viewer.cameraControl.panInertia = 0;

        }
    }

    /**
     * 是否启用旋转功能
     * @param enable true是启用,false为禁用
     */
    enableRotation(enable: boolean = true): void {
        if (enable) {
            this.viewer.cameraControl.dragRotationRate = 360;
            this.viewer.cameraControl.keyboardDollyRate = 90;
        } else {
            this.viewer.cameraControl.dragRotationRate = 0;
            this.viewer.cameraControl.keyboardDollyRate = 0;
        }
    }

    /**
     * 是否启用缩放功能
     * @param enable true是启用,false为禁用
     */
    enableScaling(enable: boolean = true): void {
        if (enable) {
            this.viewer.cameraControl.mouseWheelDollyRate = 100;
            this.viewer.cameraControl.keyboardDollyRate = 15;
            this.viewer.cameraControl.touchDollyRate = 0.2;
        } else {
            this.viewer.cameraControl.mouseWheelDollyRate = 0;
            this.viewer.cameraControl.keyboardDollyRate = 0;
            this.viewer.cameraControl.touchDollyRate = 0;
        }
    }

}
