import {BstLoader} from './loaders/bst-loader';
import {NavCubePlugin, Viewer} from '@xeokit/xeokit-sdk';
import {BstCamera} from './camera';
import {BstObjectTree} from './object-tree';
import {BstObject} from './object';
import {BstSnapShot} from './snapshot';
import {BstEvent} from './event';
import {BstMarker} from './marker';
import {BstMeasurement} from './measurement';
import {BstSlice} from './slice';

export {BstCamera} from './camera';
export {BstLoader} from './loaders/bst-loader';
export {BstObject} from './object'
export {BstObjectTree} from './object-tree';
export {BstSnapShot} from './snapshot'
export {BstEvent} from './event'
export {BstMarker} from './marker'

/**
 * Api服务统一入口服务类
 * 只需要实例化本类,本类包含了所有其他功能的引用
 */
export class BimShowTime {
    event: BstEvent;
    viewer: Viewer;
    state: { mode: string | null, marqueeFit: boolean } = {
        mode: null,
        marqueeFit: false
    }
    bstMarker: BstMarker;
    private readonly type: string = '3d';
    loadedModels: Array<{ id: string, path: string }> = new Array<{ id: string; path: string }>();
    private box: HTMLElement;
    private navCubeCanvas: HTMLCanvasElement;
    sliceCanvas: HTMLCanvasElement;
    private navCube: NavCubePlugin;

    constructor(domId: string, type: string = '3d') {
        this.type = type;
        let canvasBox = document.getElementById(domId);
        this.box = canvasBox;
        if (canvasBox) {
            canvasBox.innerHTML = '';
            let canvas = document.createElement('canvas');
            canvas.style.width = '100%';
            canvas.style.height = '100%';
            canvasBox.appendChild(canvas)
            this.viewer = new Viewer({
                canvasElement: canvas,
                transparent: true
            });
            this.event = new BstEvent(this);
            if (!this.is2D) {
                this.navCubeCanvas = document.createElement('canvas');
                this.navCubeCanvas.style.width = '200px';
                this.navCubeCanvas.style.height = '200px';
                this.navCubeCanvas.style.bottom = '50px';
                this.navCubeCanvas.style.left = '50px';
                this.navCubeCanvas.style.zIndex = '200000';
                this.navCubeCanvas.style.position = 'absolute';
                canvasBox.appendChild(this.navCubeCanvas)
                this.sliceCanvas = document.createElement('canvas');
                this.sliceCanvas.style.width = '200px';
                this.sliceCanvas.style.height = '200px';
                this.sliceCanvas.style.bottom = '50px';
                this.sliceCanvas.style.right = '50px';
                this.sliceCanvas.style.zIndex = '200000';
                this.sliceCanvas.style.position = 'absolute';
                this.sliceCanvas.id = 'slice-' + this.generateRandomString(10);
                canvasBox.appendChild(this.sliceCanvas)
                this.navCube = new NavCubePlugin(this.viewer, {
                    canvasElement: this.navCubeCanvas,
                    color: 'lightblue',
                    visible: true,           // Initially visible (default)
                    cameraFly: 'true',       // Fly camera to each selected axis/diagonal
                    cameraFitFOV: '45',        // How much field-of-view the scene takes once camera has fitted it to view
                    cameraFlyDuration: '0.5' // How long (in seconds) camera takes to fly to each new axis/diagonal
                });
            }
            this.bstMarker = new BstMarker(this);
        } else {
            throw Error('Element Not Fount: id: ' + domId);
        }
    }

    /**
     * 切换编辑器的模式
     * @param mode 模式
     * null 为取消所有模式,注意就是null,不是字符串的null
     * 'slice' 剖切模式
     * 'angleMode' 角度测量模式
     * 'distanceMode' 距离测量模式
     * 'mark' 批注模式
     */
    public changeMode(mode: string | null = null) {
        switch (mode) {
            case 'slice':
                this.slice.enter();
                break;
            case 'angleMode':
                this.measurement.angleMode();
                break;
            case 'distanceMode':
                this.measurement.distanceMode();
                break;
            case 'mark':
                break;
            case null:
                this.slice.exit();
                this.measurement.exit();
                break;
            default:
                console.log('未知的Mode: ' + mode);
        }
    }

    /**
     * 隐藏/显示导航方块
     * @param enable true为显示,false为隐藏
     */
    public enableNavCube(enable: boolean) {
        this.navCube.setVisible(enable);
    }

    /**
     * 是否启用框选放大
     * @param enable true为启用,false为禁用
     */
    public enableMarqueeFit(enable: boolean) {
        this.state.marqueeFit = enable;
    }

    /**
     * 初始化的浏览器类型是否为2D
     */
    get is2D() {
        return this.type === '2d';
    }

    /**
     * 获取浏览器的背景颜色
     */
    getBackgroundColor(): string {
        return this.box.style.backgroundColor
    }

    /**
     * 设置浏览器的背景颜色
     * @param color 颜色16禁止 #ffffff, transparent可以透明
     */
    setBackgroundColor(color: string): void {
        this.box.style.backgroundColor = color;
    }

    /**
     * 剖切组件
     */
    get slice(): BstSlice {
        return new BstSlice(this);
    }

    /**
     * 模型加载器
     */
    get loader() {
        return new BstLoader(this);
    }

    /**
     * 相机相关操作
     */
    get camera() {
        return this.is2D ? undefined : new BstCamera(this)
    }

    /**
     * 模型构件树相关操作
     */
    get tree() {
        return this.is2D ? undefined : new BstObjectTree(this);
    }

    /**
     * 构件相关操作
     */
    get object() {
        return this.is2D ? undefined : new BstObject(this);
    }

    /**
     * 快照相关操作
     */
    get snapshot(): BstSnapShot {
        return new BstSnapShot(this);
    }

    /**
     * 批注相关操作
     */
    get marker(): BstMarker {
        return this.bstMarker;
    }

    /**
     * 测量工具
     */
    get measurement(): BstMeasurement {
        return new BstMeasurement(this);
    }

    generateRandomString(len: number) {
        const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
        let result = '';

        for (let i = 0; i < length; i++) {
            const randomIndex = Math.floor(Math.random() * characters.length);
            result += characters.charAt(randomIndex);
        }

        return result;
    }
}
